@@ -78,8 +78,6 @@ on_name_lost (GDBusConnection *connection,
               const char *name,
               gpointer    data)
 {
-        GsmManager *manager = (GsmManager *)data;
-
         if (connection == NULL) {
                 g_warning ("Lost name on bus: %s", name);
                 gsm_fail_whale_dialog_we_failed (TRUE, TRUE, NULL);
@@ -311,7 +309,7 @@ main (int argc, char **argv)
 
         debug_string = g_getenv ("GNOME_SESSION_DEBUG");
         if (debug_string != NULL) {
-                debug = rpmatch (debug_string) == TRUE || atoi (debug_string) == 1;
+                debug = atoi (debug_string) == 1;
         }
 
         error = NULL;
@@ -410,11 +408,6 @@ main (int argc, char **argv)
                 g_free (ibus_path);
         }
 
-        /* Some third-party programs rely on GNOME_DESKTOP_SESSION_ID to
-         * detect if GNOME is running. We keep this for compatibility reasons.
-         */
-        gsm_util_setenv ("GNOME_DESKTOP_SESSION_ID", "this-is-deprecated");
-
         /* We want to use the GNOME menus which has the designed categories.
          */
         gsm_util_setenv ("XDG_MENU_PREFIX", "gnome-");
